@@ -27,7 +27,7 @@ import six
 
 import oslo_messaging
 from oslo_messaging._i18n import _
-from oslo_messaging._i18n import _LE
+from oslo_messaging._i18n import _LE, _LW
 from oslo_messaging import _utils as utils
 
 LOG = logging.getLogger(__name__)
@@ -229,7 +229,9 @@ def deserialize_remote_exception(data, allowed_remote_exmods):
             raise TypeError("Can only deserialize Exceptions")
 
         failure = klass(*failure.get('args', []), **failure.get('kwargs', {}))
-    except (AttributeError, TypeError, ImportError):
+    except (AttributeError, TypeError, ImportError) as error:
+        LOG.warning(_LW("Failed to rebuild remote exception due to error: %s"),
+                    six.text_type(error))
         return oslo_messaging.RemoteError(name, failure.get('message'), trace)
 
     ex_type = type(failure)
@@ -347,6 +349,9 @@ class DecayingTimer(object):
     def start(self):
         self._watch.start()
 
+    def restart(self):
+        self._watch.restart()
+
     def check_return(self, timeout_callback=None, *args, **kwargs):
         maximum = kwargs.pop('maximum', None)
         left = self._watch.leftover(return_none=True)
@@ -421,16 +426,18 @@ class ConnectionContext(Connection):
                     LOG.exception(_LE("Fail to reset the connection, drop it"))
                     try:
                         self.connection.close()
-                    except Exception:
-                        pass
+                    except Exception as exc:
+                        LOG.debug("pooled conn close failure (ignored): %s",
+                                  str(exc))
                     self.connection = self.connection_pool.create()
                 finally:
                     self.connection_pool.put(self.connection)
             else:
                 try:
                     self.connection.close()
-                except Exception:
-                    pass
+                except Exception as exc:
+                    LOG.debug("pooled conn close failure (ignored): %s",
+                              str(exc))
             self.connection = None
 
     def __exit__(self, exc_type, exc_value, tb):
