@@ -194,13 +194,6 @@ rabbit_opts = [
                default=2,
                help='How often times during the heartbeat_timeout_threshold '
                'we check the heartbeat.'),
-
-    # NOTE(sileht): deprecated option since oslo_messaging 1.5.0,
-    cfg.BoolOpt('fake_rabbit',
-                default=False,
-                deprecated_group='DEFAULT',
-                help='Deprecated, use rpc_backend=kombu+memory or '
-                'rpc_backend=fake'),
 ]
 
 LOG = logging.getLogger(__name__)
@@ -463,7 +456,6 @@ class Connection(object):
         self.interval_max = driver_conf.rabbit_interval_max
 
         self.login_method = driver_conf.rabbit_login_method
-        self.fake_rabbit = driver_conf.fake_rabbit
         self.virtual_host = driver_conf.rabbit_virtual_host
         self.rabbit_hosts = driver_conf.rabbit_hosts
         self.rabbit_port = driver_conf.rabbit_port
@@ -501,12 +493,7 @@ class Connection(object):
             virtual_host = self.virtual_host
 
         self._url = ''
-        if self.fake_rabbit:
-            LOG.warning(_LW("Deprecated: fake_rabbit option is deprecated, "
-                            "set rpc_backend to kombu+memory or use the fake "
-                            "driver instead."))
-            self._url = 'memory://%s/' % virtual_host
-        elif url.hosts:
+        if url.hosts:
             if url.transport.startswith('kombu+'):
                 LOG.warning(_LW('Selecting the kombu transport through the '
                                 'transport url (%s) is a experimental feature '
@@ -750,7 +737,7 @@ class Connection(object):
                         else interval)
 
             info = {'err_str': exc, 'sleep_time': interval}
-            info.update(self._get_connection_info())
+            info.update(self._get_connection_info(conn_error=True))
 
             if 'Socket closed' in six.text_type(exc):
                 LOG.error(_LE('[%(connection_id)s] AMQP server'
@@ -761,8 +748,7 @@ class Connection(object):
                 LOG.error(_LE('[%(connection_id)s] AMQP server on '
                               '%(hostname)s:%(port)s is unreachable: '
                               '%(err_str)s. Trying again in '
-                              '%(sleep_time)d seconds. Client port: '
-                              '%(client_port)s'), info)
+                              '%(sleep_time)d seconds.'), info)
 
             # XXX(nic): when reconnecting to a RabbitMQ cluster
             # with mirrored queues in use, the attempt to release the
@@ -1154,10 +1140,15 @@ class Connection(object):
         with self._connection_lock:
             self.ensure(method, retry=retry, error_callback=_error_callback)
 
-    def _get_connection_info(self):
+    def _get_connection_info(self, conn_error=False):
+        # Bug #1745166: set 'conn_error' true if this is being called when the
+        # connection is in a known error state.  Otherwise attempting to access
+        # the connection's socket while it is in an error state will cause
+        # py-amqp to attempt reconnecting.
         info = self.connection.info()
         client_port = None
-        if (self.channel and hasattr(self.channel.connection, 'sock')
+        if (not conn_error and self.channel
+                and hasattr(self.channel.connection, 'sock')
                 and self.channel.connection.sock):
             client_port = self.channel.connection.sock.getsockname()[1]
         info.update({'client_port': client_port,
